#ifndef BALLS_H
#define BALLS_H

#ifndef _BE_H
#error Need the precompiled header BeHeaders.pch++ !
#endif

#include "PixelBitmap.h"

// This removes experimental and debugging features
#define RELEASE

#ifndef RELEASE
#	define DEBUG 1
#endif

#include <support/Debug.h>

// if this is defined, the balls are stored in
// a "hollow" array instead of being always
// contiguous. This allows removing a particular
// ball. There are still untracked bugs though.
//#define HOLES

// if this is defined, the window is locked for
// the whole Tick() call.
//#define LOCKOUTSIDE

// if this is defined, keydowns get purged from
// the message queue.
//#define EATKEYS

// if this is defined, the balls have coordinates
// relative to the screen instead of the view.
// This doesn't look good because I can't prevent
// the app_server from copying the view's current
// contents when it's moved - it would look good if
// the app_server just left the screen like it is.
//#define SCREEN

class BallsApp : public BApplication
{
public:
	BallsApp();

	virtual void AboutRequested();
};

class BallsWindow : public BWindow
{
public:
	BallsWindow(BRect frame);
	virtual bool QuitRequested();
#ifdef SCREEN
	virtual void FrameMoved(BPoint screenpoint);
#endif
};

struct Ball
{
	BPoint pos;
	float ix,iy;
	int target;
#ifdef HOLES
	bool used;
	bool use_delayed;
#endif
};

const int n_balls_max=3000;

class BallsView : public BView
{
private:
	
	int n_balls;
	int n_visible;
	
	PixelSprite *the_sprite;
	
	Ball balls[n_balls_max];
	
#ifdef HOLES
	int first_unused;
	int first_used;
	int last_used;
	
	bool trans;
	int first_in_trans;
	int last_in_trans;
#endif
	
	thread_id Ticker_id;
	
	PixelBitmap *the_bitmap;
	
	BRect fBounds;
	
#ifdef SCREEN
	BRect fScreenFrame;
#endif
	
	bool back;
	bool bench;
	
#ifdef HOLES
	void BeginTrans();
	void Commit();	
	int  AddBall();
	void AddBalls(int howmany);
	void RemoveBall();
	void RemoveBall(int index);
	void RemoveBalls(int howmany);
#endif
	
public:

	double start_time;
	double last_calc;
	double last_frame;

	BallsView(BRect frame);
	~BallsView();
	
	void RefreshTitle();
	
	void Tick();
	
#ifdef SCREEN
	void WindowMoved();
#endif

	virtual void AttachedToWindow();
	virtual void FrameResized(float w,float h);
	virtual void DetachedFromWindow();
	virtual void Draw(BRect update);
	virtual void KeyDown(ulong achar);
	virtual void MouseDown(BPoint point);
#ifdef EATKEYS
	virtual void MessageReceived(BMessage *message);
#endif

//PC void DrawBitmap(PixelBitmap *bitmap,BRect source,BRect dest);
};

#ifdef HOLES

#define FOR_EACH_BALL(counter) \
	for (int counter=first_used;counter<=last_used;counter++) \
	if (balls[counter].used)

#define FOR_EACH_BALL_TRANS(counter) \
	for (int counter=first_used;counter<=last_used;counter++) \
	if (balls[counter].used && !balls[counter].use_delayed)

#else

#define FOR_EACH_BALL(counter) \
	for (int counter=0;counter<n_balls;counter++)
	
#endif

const float frame=30000.0;

#endif // BALLS_H